function [BW3dBa BW3dBe PeakFa PeakFe Qa Qe Levela Levele Asymma Asymme]=resprocessSA(B,numswp,freq,RBW,Transmiss)
% Algorithm for calculating Bandwidth, Peak Frequency, Q, Amplitude, and
% Asymmetry of resonances measured on the spectrum analyzer in a sweep-on-scan mode.

% Inputs 
% B = Amplitude data in a matrix format with each sweep stored in its own column
% numswp = number of sweeps (typically 5)
% freq = 601-point vector of frequency values corresponding to B
% RBW = Resolution BandWidth of the spectrum analyzer during measurement
% Transmiss = 601-point vector of transmissivity values used for
% deconvolution

% Outputs
% BW3dBa BW3dBe = 3dB bandwidth mean and 2-sigma standard deviation, each
% with three values: the normal width, twice the higher half-width, and
% twice the lower half-width
% PeakFa PeakFe = peak frequency mean and 2-sigma standard deviation
% Qa Qe = Q mean and 2-sigma standard deviation, each with three values: 
% the normal Q, twice the higher half-width, and twice the lower half-width
% Levela Levele = amplitude mean and 2-sigma standard deviation
% Asymma Asymme = asymmetry mean and 2-sigma standard deviation

Boriginal=B; % stores original B data

fix1=zeros(1,numswp); % variable for tracking number of iterations for up smoothing
fix2=zeros(1,numswp); % variable for tracking number of iterations for down smoothing
fix3=zeros(1,numswp); % variable for tracking number of iterations for extra smoothing
nofa=0; % variable used if low frequency 3-dB point is not found
nofb=0; % variable used if high frequency 3-dB point is not found
maxfixup=15; % set max number of up smoothing iterations
maxfixdown=10; % set max number of down smoothing iterations
maxfixsmooth=5; % set max number of extra smoothing iterations
uptol=0.0001; % set up smoothing tolerance
downtol=0.05; % set down smoothing tolerance
smoothtol=0.001; % set extra smoothing tolerance

bottom=min(B(:,1)); % find bottom value of measurement
Bo=ones(300,numswp)*bottom; % variable to store real amplitude points from the sweeps and not nulls
freqo=ones(300,numswp)*freq(601); % variable to store corresponding frequency points
x=5; % initial smoothing enhancement exponent
Pass=0; 
while Pass<1 % loop for finding optimum smoothing factor
for s=1:numswp % smooth and calculate statistics for each sweep individually
    clear Po
    tol=1; % define tolerance
    diff=zeros(601,numswp);
    
    i=1;
    while B(i,s)==bottom
        i=i+1;
    end
    startf=i; % find point at which lowest frequency good data occurs
    i=601;
    while B(i,s)==bottom
        i=i-1;
    end
    stopf=i; % find point at which highest frequency good data occurs
    [peak(s), location(s)]=max(B(:,s));
    

while tol>uptol % loop for up smoothing low data points (nulls)
    % four-point averaging filter
    avg(1)=(B(2,s)+B(3,s))/2;
    avg(2)=(B(1,s)+B(3,s)+B(4,s))/3;
    for k=3:599
        avg(k)=(B(k-2,s)+B(k-1,s)+B(k+1,s)+B(k+2,s))/4;
    end
    avg(601)=(B(599,s)+B(600,s))/2;
    avg(600)=(B(598,s)+B(599,s)+B(601,s))/3;
    
    for m=startf:stopf 
        if B(m,s)< avg(m)
            if avg(m)==0
                diff(m,s)=0;
            else
                diff(m,s)=abs((-avg(m)+B(m,s))/avg(m)); % find percentage difference 
            end
            B(m,s)=avg(m); % raise the value of the point to the new average
        end
    end
    tol=max(diff(:,s)); % calculate new tolerance   
    if fix1(s)==maxfixup
        tol=0; % stop smoothing due to maximum iterations
    else
        fix1(s)=fix1(s)+1;
    end
end
B1(:,s)=B(:,s); % save what B looks like after upsmoothing
tol=1;
diff(:,s)=zeros(601,1);
while tol>downtol % loop for down smoothing high data points (spikes)
    % four-point averaging filter    
    avg(1)=(B(2,s)+B(3,s))/2;
    avg(2)=(B(1,s)+B(3,s)+B(4,s))/3;
    for k=3:599
        avg(k)=(B(k-2,s)+B(k-1,s)+B(k+1,s)+B(k+2,s))/4;
    end
    avg(601)=(B(599,s)+B(600,s))/2;
    avg(600)=(B(598,s)+B(599,s)+B(601,s))/3;
    for m=startf:stopf
        if B(m,s)> avg(m)
            if avg(m)==0
                diff(m,s)=0;
            else
                diff(m,s)=abs((avg(m)-B(m,s))/avg(m)); % find percentage difference
            end
            B(m,s)=avg(m); % lower the value of the point to the new average
        end
    end
    tol=max(diff(:,s));
    if fix2(s)==maxfixdown
        tol=0; % stop smoothing due to maximum iterations
    else
        fix2(s)=fix2(s)+1;
    end
end
B2(:,s)=B(:,s); % save what B looks like after upsmoothing and downsmoothing
tol=1;
diff(:,s)=zeros(601,1);
while tol>smoothtol % loop for additional smoothing
    % two-point averaging filter
    avg(1)=B(2,s);
    for k=2:600
        avg(k)=(B(k-1,s)+B(k+1,s))/2;
    end
    avg(601)=B(600,s);
    for m=startf:stopf
        if avg(m)==0
            diff(m,s)=0;
        else
            diff(m,s)=abs((avg(m)-B(m,s))/avg(m)); % find percentage difference
        end
        B(m,s)=avg(m); % set the value of the point to the new average
    end
    tol=max(diff(:,s));
    if fix3(s)==maxfixsmooth
        tol=0; % stop smoothing due to maximum iterations
    else
        fix3(s)=fix3(s)+1;
    end
end

% deconvolve effect of transmissivity zeroed to the peak of the resonance
[peak(s), location2(s)]=max(B(:,s)); 
ZeroVal=Transmiss(location2(s)).*ones(601,1); 
Boriginal2(:,s)=Boriginal(:,s)-Transmiss+ZeroVal;
[peak(s), location2(s)]=max(Boriginal2(:,s)); 

% find which of the original 601 points are valid data and place them into
% new matrices
n=0;
for q=startf:stopf
    if Boriginal(q,s)~=bottom && (B(q,s)-Boriginal(q,s))<0.1
        n=n+1;
        Bo(n,s)=Boriginal2(q,s);
        Bo2(n)=Boriginal2(q,s);
        freqo(n,s)=freq(q);
        freqo2(n)=freq(q);
    end
end
[peak(s), location2(s)]=max(Bo(:,s));
fre=linspace(freq(startf),freq(stopf), 10001);
[pp, p]=csaps(freqo2,Bo2); % run cubic smoothing spline to determine default tolerance
[pp2, p2]=csaps(freqo2,Bo2,p*10^(-x)); % run spline with enhanced tolerance
Po=ppval(pp2,fre); % create a 10001 point smoothed version of the data

% create a smoothed data set equal in length to the original "good" data
% and save it for each sweep
Ponumpoints(:,s)=ppval(pp2, freqo(:,s)); 

% create another smoothed data set equal in length to the original "good" data
Po2=ppval(pp2,freqo2); 
[peak(s), location(s)]=max(Po); % find peak amplitude
threshold(s)=ppval(pp2,fre(location(s)))-3.0103; % find upper and lower half-power points
PeakF(s)=fre(location(s)); % find peak frequency with high resolution

% iterate through the data to find the frequencies of the upper and lower
% half-power points and use a linear interpolation for even more precision 
q=location(s);
while Po(q)>threshold(s)
    q=q+1;
    if q==10001
        disp('upper 3dB point not found, extrapolating...')
        nofb=1;
        break 
    end
end
if nofb==1
    dfb=(Po(10001)-threshold(s))*(fre(10001)-fre(10000))/(Po(10000)-Po(10001));
    fb(s)=fre(10001)+dfb;
else
    fb(s)=(Po(q-1)-threshold(s))*(fre(q)-fre(q-1))/(Po(q-1)-Po(q))+fre(q-1);
end

q=location(s);
while Po(q)>threshold(s)
    q=q-1;
    if q==0;
        disp('lower 3dB point not found, extrapolating...')
        nofa=1;
        break
    end
end
if nofa==1
    dfa=(Po(1)-threshold(s))*(fre(2)-fre(1))/(Po(2)-Po(1));
    fa(s)=fre(1)-dfa;
else
    fa(s)=fre(q+1)-(Po(q+1)-threshold(s))*(fre(q+1)-fre(q))/(Po(q+1)-Po(q));
end

bw3db(s)=sqrt((fb(s)-fa(s))^2-(RBW)^2);
bw3dbh(s)=sqrt((2*(fb(s)-PeakF(s)))^2-RBW^2); % twice the high-frequency side half bandwidth
bw3dbl(s)=sqrt((2*(PeakF(s)-fa(s)))^2-RBW^2); % twice the low-frequency side half bandwidth
Q(s)=PeakF(s)/bw3db(s);
Qh(s)=PeakF(s)/bw3dbh(s); % high-frequency side Q
Ql(s)=PeakF(s)/bw3dbl(s); % low-frequency side Q
asym(s)=100*((fb(s)-PeakF(s))-(PeakF(s)-fa(s)))/(fb(s)-fa(s)); % asymmetry calculation (from DeBoer and Steffes 1996)

end

% find the average amount that the smoothed curve differs from the raw data
% at the peak and 5 points on either side and lower the smoothing
% enhancement factor if the difference is too great
avg=mean(Ponumpoints,2);
avg2=mean(Bo,2);
q=round(mean(location2));
for y=q-5:q+5
    diff(y-q+6)=avg(y)-avg2(y);
end    

if mean(peak)-bottom>10 % use for dielectrically matched measurements where resolution is 2 dB/div
    if abs(mean(diff))<0.0002
        Pass=1;
    else
        x=x-1 % display new smoothing enhancement factor for user to get a 
          % feel for how rough the data really is
    end
else % resolution = 1 dB/div
    if abs(mean(diff))<0.0001
        Pass=1;
    else
        x=x-1 % display new smoothing enhancement factor for user to get a 
          % feel for how rough the data really is
    end
end
end
% calculates the mean and sample standard deviation of the values to return
BW3dBa=[mean(bw3db); mean(bw3dbh); mean(bw3dbl)];
BW3dBe=[std(bw3db); std(bw3dbh); std(bw3dbl)];
PeakFa=mean(PeakF);
PeakFe=std(PeakF);
Qa=[mean(Q); mean(Qh); mean(Ql)];
Qe=[std(Q); std(Qh); std(Ql)];
Levela=mean(peak);
Levele=std(peak);
Asymma=mean(asym);
Asymme=std(asym);
