function [BW3dBa BW3dBe PeakFa PeakFe Qa Qe Levela Levele Asymma Asymme]=resprocessSAsync(B,numswp,freq,RBW,Transmiss)

% Algorithm for processing resonances taken on a synchronized spectrum analyzer or other 
% device that does not have any sweep on scan nulls

% It compares the effect of the smoothing to the original data to make sure
% that the data isn't over smoothed, which would decrease the amplitude of
% the peak.  This algorithm tries highly smoothed values first and
% decreases by a factor of 10 each iteration until the absolute difference
% of the average of the smoothed versus unsmoothed peak values is less than
% 0.01 dB.

% Inputs 
% B = Amplitude data in a matrix format with each sweep stored in its own column
% numswp = number of sweeps (typically 10)
% freq = 601-point vector of frequency values corresponding to B
% Transmiss = 601-point vector of transmissivity values used for
% deconvolution, if necessary

% Outputs
% BW3dBa BW3dBe = 3dB bandwidth mean and 2-sigma standard deviation, each
% with three values: the normal width, twice the higher half-width, and
% twice the lower half-width
% PeakFa PeakFe = peak frequency mean and 2-sigma standard deviation
% Qa Qe = Q mean and 2-sigma standard deviation, each with three values: 
% the normal Q, twice the higher half-width, and twice the lower half-width
% Levela Levele = amplitude mean and 2-sigma standard deviation
% Asymma Asymme = asymmetry mean and 2-sigma standard deviation

Boriginal=B;  % maintains original data
nofa=0;  
% marker for resonances whose lower -3dB point is out of the range of data

nofb=0;  
% marker for resonances whose higher -3dB point is out of the range of data

x=5;    % original (usually overly smooth) smoothing enhancement factor
Pass=0;
while Pass<1
for s=1:numswp   % perform smoothing on each sweep independently
[peak(s), location2(s)]=max(B(:,s)); 
location(s)=location2(s);

ZeroVal=Transmiss(location(s)).*ones(length(freq),1);
% pins the amplitude of the transmissivity deconvolution to zero at the
% peak of the resonance
 
B(:,s)=B(:,s)-Transmiss+ZeroVal;
% deconvolves the effect of transmissivity of the cables
 

fre=linspace(freq(1),freq(length(freq)), 10001);  
% creates a longer, more precise, frequency vector

[pp, p]=csaps(freq,B(:,s));  
% smoothing algorithm with default value (usually too course)

[pp, p2]=csaps(freq,B(:,s),p*10^(-x)); 
% enhanced smoothing algorithm with new value

Po=ppval(pp,fre);  % smoothed 10,001 point version of B(:,s)

Ponumpoints(:,s)=ppval(pp, freq); 
% smoothed version of B(:,s) with the original number of points

[peak(s), location(s)]=max(Po);
threshold(s)=ppval(pp,fre(location(s)))-3.0103; % find upper and lower half-power points
PeakF(s)=fre(location(s));
q=location(s);
while Po(q)>threshold(s) % find upper -3dB point
    q=q+1;
    if q==10001
        disp('upper -3dB point not found, interpolating...')
        nofb=1;
        break 
    end
end
if nofb==1
    dfb=(Po(10001)-threshold(s))*(fre(10001)-fre(10000))/(Po(10000)-Po(10001));
    fb(s)=fre(10001)+dfb; % linear extrapolation out to -3dB point
else
    fb(s)=(Po(q-1)-threshold(s))*(fre(q)-fre(q-1))/(Po(q-1)-Po(q))+fre(q-1);
    % linear interpolation of -3dB point
end

q=location(s);
while Po(q)>threshold(s) % find lower -3dB point
    q=q-1;
    if q==0;
        disp('lower -3dB point not found, interpolating...')
        nofa=1;
        break
    end
end
if nofa==1
    dfa=(Po(1)-threshold(s))*(fre(2)-fre(1))/(Po(2)-Po(1));
    fa(s)=fre(1)-dfa; % linear extrapolation out to -3dB point
else
    fa(s)=fre(q+1)-(Po(q+1)-threshold(s))*(fre(q+1)-fre(q))/(Po(q+1)-Po(q));
    % linear interpolation of -3dB point
end

bw3db(s)=sqrt((fb(s)-fa(s))^2-(RBW)^2); % calculate 3dB bandwidth

% used in calculating asymmetry uncertainty
bw3dbh(s)=sqrt((2*(fb(s)-PeakF(s)))^2-RBW^2); 
% bandwidth if doubling higher frequency half BW

bw3dbl(s)=sqrt((2*(PeakF(s)-fa(s)))^2-RBW^2);
% bandwidth if doubling lower frequency half BW

Q(s)=PeakF(s)/bw3db(s);
Qh(s)=PeakF(s)/bw3dbh(s);
Ql(s)=PeakF(s)/bw3dbl(s);
asym(s)=100*((fb(s)-PeakF(s))-(PeakF(s)-fa(s)))/(fb(s)-fa(s));
% from DeBoer and Steffes, 1996
end
avg=mean(Ponumpoints,2);
avg2=mean(Boriginal,2);
y=round(mean(location2));

if length(freq)==1601
    toler=0.01;
elseif length(freq)==401
    toler=0.02;
elseif length(freq)==601
    toler=0.02;    
end

if abs(avg(y)-avg2(y))<toler
    Pass=1;
else
    x=x-1 % display new smoothing enhancement factor for user to get a feel
          % for how rough the data really is
end
end

% calculates the mean and sample standard deviation of the values to return
BW3dBa=[mean(bw3db); mean(bw3dbh); mean(bw3dbl)];
BW3dBe=[std(bw3db); std(bw3dbh); std(bw3dbl)];
PeakFa=mean(PeakF);
PeakFe=std(PeakF);
Qa=[mean(Q); mean(Qh); mean(Ql)];
Qe=[std(Q); std(Qh); std(Ql)];
Levela=mean(peak);
Levele=std(peak);
Asymma=mean(asym);
Asymme=std(asym);